try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class ViewType(object):
	def __init__(self):
		"""Creates an instance of ViewType"""

		self.__view = None
		self.__edit = None
		self.__create = None
		self.__quick_create = None
		self.__key_modified = dict()

	def get_view(self):
		"""
		The method to get the view

		Returns:
			bool: A bool representing the view
		"""

		return self.__view

	def set_view(self, view):
		"""
		The method to set the value to view

		Parameters:
			view (bool) : A bool representing the view
		"""

		if view is not None and not isinstance(view, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: view EXPECTED TYPE: bool', None, None)
		
		self.__view = view
		self.__key_modified['view'] = 1

	def get_edit(self):
		"""
		The method to get the edit

		Returns:
			bool: A bool representing the edit
		"""

		return self.__edit

	def set_edit(self, edit):
		"""
		The method to set the value to edit

		Parameters:
			edit (bool) : A bool representing the edit
		"""

		if edit is not None and not isinstance(edit, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: edit EXPECTED TYPE: bool', None, None)
		
		self.__edit = edit
		self.__key_modified['edit'] = 1

	def get_create(self):
		"""
		The method to get the create

		Returns:
			bool: A bool representing the create
		"""

		return self.__create

	def set_create(self, create):
		"""
		The method to set the value to create

		Parameters:
			create (bool) : A bool representing the create
		"""

		if create is not None and not isinstance(create, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: create EXPECTED TYPE: bool', None, None)
		
		self.__create = create
		self.__key_modified['create'] = 1

	def get_quick_create(self):
		"""
		The method to get the quick_create

		Returns:
			bool: A bool representing the quick_create
		"""

		return self.__quick_create

	def set_quick_create(self, quick_create):
		"""
		The method to set the value to quick_create

		Parameters:
			quick_create (bool) : A bool representing the quick_create
		"""

		if quick_create is not None and not isinstance(quick_create, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: quick_create EXPECTED TYPE: bool', None, None)
		
		self.__quick_create = quick_create
		self.__key_modified['quick_create'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
    from zcrmsdk.src.com.zoho.crm.api.util.header_param_validator import HeaderParamValidator
    from zcrmsdk.src.com.zoho.crm.api.header import Header
    from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
    from zcrmsdk.src.com.zoho.crm.api.util.constants import Constants
except Exception:
    from .util import HeaderParamValidator
    from .header import Header
    from ..api.exception import SDKException
    from ..api.util import Constants


class HeaderMap(object):

    """
    This class represents the HTTP header name and value.
    """

    def __init__(self):
        """Creates an instance of HeaderMap Class"""

        self.request_headers = dict()

    def add(self, header, value):

        """
        The method to add the parameter name and value.

        Parameters:
            header (Header): A Header class instance.
            value (object): An object containing the header value.
        """

        if header is None:
            raise SDKException(Constants.HEADER_NONE_ERROR, Constants.HEADER_INSTANCE_NONE_ERROR)

        header_name = header.name

        if header_name is None:
            raise SDKException(Constants.HEADER_NAME_NONE_ERROR, Constants.HEADER_NAME_NULL_ERROR_MESSAGE)

        if value is None:
            raise SDKException(Constants.HEADER_NONE_ERROR, header_name + Constants.NONE_VALUE_ERROR_MESSAGE)

        class_name = header.class_name

        if class_name is not None:
            value = HeaderParamValidator().validate(header, value)

        if header_name not in self.request_headers:
            self.request_headers[header_name] = str(value)

        else:
            header_value = self.request_headers[header_name]
            self.request_headers[header_name] = header_value + ',' + str(value)



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Record(object):
	def __init__(self):
		"""Creates an instance of Record"""

		self.__key_values = dict()
		self.__key_modified = dict()

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.get_key_value('id')

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.add_key_value('id', id)

	def get_created_by(self):
		"""
		The method to get the created_by

		Returns:
			User: An instance of User
		"""

		return self.get_key_value('Created_By')

	def set_created_by(self, created_by):
		"""
		The method to set the value to created_by

		Parameters:
			created_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if created_by is not None and not isinstance(created_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by EXPECTED TYPE: User', None, None)
		
		self.add_key_value('Created_By', created_by)

	def get_created_time(self):
		"""
		The method to get the created_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.get_key_value('Created_Time')

	def set_created_time(self, created_time):
		"""
		The method to set the value to created_time

		Parameters:
			created_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if created_time is not None and not isinstance(created_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_time EXPECTED TYPE: datetime', None, None)
		
		self.add_key_value('Created_Time', created_time)

	def get_modified_by(self):
		"""
		The method to get the modified_by

		Returns:
			User: An instance of User
		"""

		return self.get_key_value('Modified_By')

	def set_modified_by(self, modified_by):
		"""
		The method to set the value to modified_by

		Parameters:
			modified_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if modified_by is not None and not isinstance(modified_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_by EXPECTED TYPE: User', None, None)
		
		self.add_key_value('Modified_By', modified_by)

	def get_modified_time(self):
		"""
		The method to get the modified_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.get_key_value('Modified_Time')

	def set_modified_time(self, modified_time):
		"""
		The method to set the value to modified_time

		Parameters:
			modified_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if modified_time is not None and not isinstance(modified_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_time EXPECTED TYPE: datetime', None, None)
		
		self.add_key_value('Modified_Time', modified_time)

	def get_tag(self):
		"""
		The method to get the tag

		Returns:
			list: An instance of list
		"""

		return self.get_key_value('Tag')

	def set_tag(self, tag):
		"""
		The method to set the value to tag

		Parameters:
			tag (list) : An instance of list
		"""

		if tag is not None and not isinstance(tag, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: tag EXPECTED TYPE: list', None, None)
		
		self.add_key_value('Tag', tag)

	def add_field_value(self, field, value):
		"""
		The method to add field value

		Parameters:
			field (Field) : An instance of Field
			value (object) : An object
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.record.field import Field
		except Exception:
			from .field import Field

		if field is not None and not isinstance(field, Field):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: field EXPECTED TYPE: Field', None, None)
		
		self.add_key_value(field.get_api_name(), value)

	def add_key_value(self, api_name, value):
		"""
		The method to add key value

		Parameters:
			api_name (string) : A string representing the api_name
			value (Object) : A Object representing the value
		"""

		if api_name is not None and not isinstance(api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: api_name EXPECTED TYPE: str', None, None)
		
		self.__key_values[api_name] = value
		self.__key_modified[api_name] = 1

	def get_key_value(self, api_name):
		"""
		The method to get key value

		Parameters:
			api_name (string) : A string representing the api_name

		Returns:
			Object: A Object representing the key_value
		"""

		if api_name is not None and not isinstance(api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: api_name EXPECTED TYPE: str', None, None)
		
		if api_name in self.__key_values:
			return self.__key_values.get(api_name)
		
		return None

	def get_key_values(self):
		"""
		The method to get key values

		Returns:
			dict: An instance of dict
		"""

		return self.__key_values

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Pipeline(object):
	def __init__(self):
		"""Creates an instance of Pipeline"""

		self.__from_1 = None
		self.__to = None
		self.__parent = None
		self.__child_available = None
		self.__display_value = None
		self.__default = None
		self.__maps = None
		self.__actual_value = None
		self.__id = None
		self.__key_modified = dict()

	def get_from(self):
		"""
		The method to get the from

		Returns:
			int: An int representing the from_1
		"""

		return self.__from_1

	def set_from(self, from_1):
		"""
		The method to set the value to from

		Parameters:
			from_1 (int) : An int representing the from_1
		"""

		if from_1 is not None and not isinstance(from_1, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: from_1 EXPECTED TYPE: int', None, None)
		
		self.__from_1 = from_1
		self.__key_modified['from'] = 1

	def get_to(self):
		"""
		The method to get the to

		Returns:
			int: An int representing the to
		"""

		return self.__to

	def set_to(self, to):
		"""
		The method to set the value to to

		Parameters:
			to (int) : An int representing the to
		"""

		if to is not None and not isinstance(to, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: to EXPECTED TYPE: int', None, None)
		
		self.__to = to
		self.__key_modified['to'] = 1

	def get_parent(self):
		"""
		The method to get the parent

		Returns:
			Pipeline: An instance of Pipeline
		"""

		return self.__parent

	def set_parent(self, parent):
		"""
		The method to set the value to parent

		Parameters:
			parent (Pipeline) : An instance of Pipeline
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.pipeline.pipeline import Pipeline
		except Exception:
			from .pipeline import Pipeline

		if parent is not None and not isinstance(parent, Pipeline):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: parent EXPECTED TYPE: Pipeline', None, None)
		
		self.__parent = parent
		self.__key_modified['parent'] = 1

	def get_child_available(self):
		"""
		The method to get the child_available

		Returns:
			bool: A bool representing the child_available
		"""

		return self.__child_available

	def set_child_available(self, child_available):
		"""
		The method to set the value to child_available

		Parameters:
			child_available (bool) : A bool representing the child_available
		"""

		if child_available is not None and not isinstance(child_available, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: child_available EXPECTED TYPE: bool', None, None)
		
		self.__child_available = child_available
		self.__key_modified['child_available'] = 1

	def get_display_value(self):
		"""
		The method to get the display_value

		Returns:
			string: A string representing the display_value
		"""

		return self.__display_value

	def set_display_value(self, display_value):
		"""
		The method to set the value to display_value

		Parameters:
			display_value (string) : A string representing the display_value
		"""

		if display_value is not None and not isinstance(display_value, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: display_value EXPECTED TYPE: str', None, None)
		
		self.__display_value = display_value
		self.__key_modified['display_value'] = 1

	def get_default(self):
		"""
		The method to get the default

		Returns:
			bool: A bool representing the default
		"""

		return self.__default

	def set_default(self, default):
		"""
		The method to set the value to default

		Parameters:
			default (bool) : A bool representing the default
		"""

		if default is not None and not isinstance(default, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: default EXPECTED TYPE: bool', None, None)
		
		self.__default = default
		self.__key_modified['default'] = 1

	def get_maps(self):
		"""
		The method to get the maps

		Returns:
			list: An instance of list
		"""

		return self.__maps

	def set_maps(self, maps):
		"""
		The method to set the value to maps

		Parameters:
			maps (list) : An instance of list
		"""

		if maps is not None and not isinstance(maps, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: maps EXPECTED TYPE: list', None, None)
		
		self.__maps = maps
		self.__key_modified['maps'] = 1

	def get_actual_value(self):
		"""
		The method to get the actual_value

		Returns:
			string: A string representing the actual_value
		"""

		return self.__actual_value

	def set_actual_value(self, actual_value):
		"""
		The method to set the value to actual_value

		Parameters:
			actual_value (string) : A string representing the actual_value
		"""

		if actual_value is not None and not isinstance(actual_value, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: actual_value EXPECTED TYPE: str', None, None)
		
		self.__actual_value = actual_value
		self.__key_modified['actual_value'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import APIResponse, CommonAPIHandler, Constants
	from zcrmsdk.src.com.zoho.crm.api.header import Header
except Exception:
	from ..exception import SDKException
	from ..util import APIResponse, CommonAPIHandler, Constants
	from ..header import Header


class ProfilesOperations(object):
	def __init__(self, if_modified_since=None):
		"""
		Creates an instance of ProfilesOperations with the given parameters

		Parameters:
			if_modified_since (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if if_modified_since is not None and not isinstance(if_modified_since, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: if_modified_since EXPECTED TYPE: datetime', None, None)
		
		self.__if_modified_since = if_modified_since


	def get_profiles(self):
		"""
		The method to get profiles

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/settings/profiles'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		handler_instance.add_header(Header('If-Modified-Since', 'com.zoho.crm.api.Profiles.GetProfilesHeader'), self.__if_modified_since)
		try:
			from zcrmsdk.src.com.zoho.crm.api.profiles.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/json')

	def get_profile(self, id):
		"""
		The method to get profile

		Parameters:
			id (int) : An int representing the id

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2.1/settings/profiles/'
		api_path = api_path + str(id)
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		handler_instance.add_header(Header('If-Modified-Since', 'com.zoho.crm.api.Profiles.GetProfileHeader'), self.__if_modified_since)
		try:
			from zcrmsdk.src.com.zoho.crm.api.profiles.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/json')


class GetProfilesHeader(object):
	pass


class GetProfileHeader(object):
	pass



try:
    import threading
    import os
    import json
    import logging
    import shutil
    from zcrmsdk.src.com.zoho.crm.api.util import Constants, Converter, Utility
    from zcrmsdk.src.com.zoho.crm.api.initializer import Initializer
    from zcrmsdk.src.com.zoho.crm.api.exception import SDKException

except Exception:
    import threading
    import os
    import json
    import logging
    import shutil
    from .constants import Constants
    from .converter import Converter
    from .utility import Utility
    from ..initializer import Initializer
    from ..exception import SDKException


class ModuleFieldsHandler(object):
    logger = logging.getLogger('SDKLogger')
    lock = threading.Lock()

    @staticmethod
    def __get_directory():

        """
        The method to obtain resources directory path.

        Returns:
            str: A String representing the directory's absolute path.
        """
        return os.path.join(Initializer.get_initializer().resource_path, Constants.FIELD_DETAILS_DIRECTORY)

    @staticmethod
    def delete_fields_file():
        """
        The method to delete fields JSON File of the current user.

        Raises:
            SDKException
        """

        with ModuleFieldsHandler.lock:
            try:
                record_field_details_path = os.path.join(ModuleFieldsHandler.__get_directory(), Converter.get_encoded_file_name())
                if os.path.exists(record_field_details_path):
                    os.remove(record_field_details_path)
            except Exception as e:
                sdk_exception = SDKException(cause=e)
                ModuleFieldsHandler.logger.info(Constants.DELETE_FIELD_FILE_ERROR + sdk_exception.__str__())
                raise sdk_exception

    @staticmethod
    def delete_all_field_files():
        """
        The method to delete all the field JSON files under resources directory.

        Raises:
            SDKException
        """

        with ModuleFieldsHandler.lock:
            try:
                record_field_details_directory = ModuleFieldsHandler.__get_directory()
                if os.path.exists(record_field_details_directory):
                    files_list = [os.path.join(record_field_details_directory, file) for file in os.listdir(record_field_details_directory) if os.path.isfile(os.path.join(record_field_details_directory, file))]
                    for each_file in files_list:
                        if Constants.JSON_FILE_EXTENSION in each_file:
                            os.remove(each_file)
            except Exception as e:
                sdk_exception = SDKException(cause=e)
                ModuleFieldsHandler.logger.info(Constants.DELETE_FIELD_FILES_ERROR + sdk_exception.__str__())
                raise sdk_exception

    @staticmethod
    def __delete_fields(module):
        """
        The method to delete fields of the given module from the current user's fields JSON file.

        Parameters:
            module(str): A string representing the module.

        Raises:
            SDKException
        """

        try:
            record_field_details_path = os.path.join(ModuleFieldsHandler.__get_directory(), Converter.get_encoded_file_name())
            if os.path.exists(record_field_details_path):
                record_field_details_json = Initializer.get_json(record_field_details_path)
                Utility.delete_fields(record_field_details_json, module)
                with open(record_field_details_path, mode="w") as file:
                    json.dump(record_field_details_json, file)
                    file.flush()
                    file.close()
        except Exception as e:
            sdk_exception = SDKException(cause=e)
            raise sdk_exception

    @staticmethod
    def refresh_fields(module):
        """
        The method to force-refresh fields of a module.

        Parameters:
            module(str): A string representing the module.

        Raises:
            SDKException
        """

        with ModuleFieldsHandler.lock:
            try:
                ModuleFieldsHandler.__delete_fields(module)
                Utility.get_fields(module)
            except SDKException as ex:
                ModuleFieldsHandler.logger.info(Constants.REFRESH_SINGLE_MODULE_FIELDS_ERROR + module + ex.__str__())
                raise ex
            except Exception as e:
                sdk_exception = SDKException(cause=e)
                ModuleFieldsHandler.logger.info(Constants.REFRESH_SINGLE_MODULE_FIELDS_ERROR + module + sdk_exception.__str__())
                raise sdk_exception

    @staticmethod
    def refresh_all_modules():
        """
        The method to force-refresh fields of all the available modules.

        Raises:
            SDKException
        """

        with ModuleFieldsHandler.lock:
            try:
                Utility.refresh_modules()
            except SDKException as ex:
                ModuleFieldsHandler.logger.info(Constants.REFRESH_ALL_MODULE_FIELDS_ERROR + ex.__str__())
                raise ex
            except Exception as e:
                sdk_exception = SDKException(cause=e)
                ModuleFieldsHandler.logger.info(Constants.REFRESH_ALL_MODULE_FIELDS_ERROR + sdk_exception.__str__())
                raise sdk_exception



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
	from zcrmsdk.src.com.zoho.crm.api.send_mail import Template
except Exception:
	from ..exception import SDKException
	from ..util import Constants
	from ..send_mail import Template


class EmailTemplate(Template):
	def __init__(self):
		"""Creates an instance of EmailTemplate"""
		super().__init__()

		self.__content = None
		self.__created_time = None
		self.__subject = None
		self.__module = None
		self.__type = None
		self.__created_by = None
		self.__modified_time = None
		self.__last_usage_time = None
		self.__associated = None
		self.__name = None
		self.__modified_by = None
		self.__description = None
		self.__id = None
		self.__editor_mode = None
		self.__favorite = None
		self.__attachments = None
		self.__folder = None
		self.__consent_linked = None
		self.__key_modified = dict()

	def get_content(self):
		"""
		The method to get the content

		Returns:
			string: A string representing the content
		"""

		return self.__content

	def set_content(self, content):
		"""
		The method to set the value to content

		Parameters:
			content (string) : A string representing the content
		"""

		if content is not None and not isinstance(content, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: content EXPECTED TYPE: str', None, None)
		
		self.__content = content
		self.__key_modified['content'] = 1

	def get_created_time(self):
		"""
		The method to get the created_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__created_time

	def set_created_time(self, created_time):
		"""
		The method to set the value to created_time

		Parameters:
			created_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if created_time is not None and not isinstance(created_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_time EXPECTED TYPE: datetime', None, None)
		
		self.__created_time = created_time
		self.__key_modified['created_time'] = 1

	def get_subject(self):
		"""
		The method to get the subject

		Returns:
			string: A string representing the subject
		"""

		return self.__subject

	def set_subject(self, subject):
		"""
		The method to set the value to subject

		Parameters:
			subject (string) : A string representing the subject
		"""

		if subject is not None and not isinstance(subject, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: subject EXPECTED TYPE: str', None, None)
		
		self.__subject = subject
		self.__key_modified['subject'] = 1

	def get_module(self):
		"""
		The method to get the module

		Returns:
			Module: An instance of Module
		"""

		return self.__module

	def set_module(self, module):
		"""
		The method to set the value to module

		Parameters:
			module (Module) : An instance of Module
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.modules import Module
		except Exception:
			from ..modules import Module

		if module is not None and not isinstance(module, Module):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: module EXPECTED TYPE: Module', None, None)
		
		self.__module = module
		self.__key_modified['module'] = 1

	def get_type(self):
		"""
		The method to get the type

		Returns:
			string: A string representing the type
		"""

		return self.__type

	def set_type(self, type):
		"""
		The method to set the value to type

		Parameters:
			type (string) : A string representing the type
		"""

		if type is not None and not isinstance(type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: type EXPECTED TYPE: str', None, None)
		
		self.__type = type
		self.__key_modified['type'] = 1

	def get_created_by(self):
		"""
		The method to get the created_by

		Returns:
			User: An instance of User
		"""

		return self.__created_by

	def set_created_by(self, created_by):
		"""
		The method to set the value to created_by

		Parameters:
			created_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if created_by is not None and not isinstance(created_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by EXPECTED TYPE: User', None, None)
		
		self.__created_by = created_by
		self.__key_modified['created_by'] = 1

	def get_modified_time(self):
		"""
		The method to get the modified_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__modified_time

	def set_modified_time(self, modified_time):
		"""
		The method to set the value to modified_time

		Parameters:
			modified_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if modified_time is not None and not isinstance(modified_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_time EXPECTED TYPE: datetime', None, None)
		
		self.__modified_time = modified_time
		self.__key_modified['modified_time'] = 1

	def get_last_usage_time(self):
		"""
		The method to get the last_usage_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__last_usage_time

	def set_last_usage_time(self, last_usage_time):
		"""
		The method to set the value to last_usage_time

		Parameters:
			last_usage_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if last_usage_time is not None and not isinstance(last_usage_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: last_usage_time EXPECTED TYPE: datetime', None, None)
		
		self.__last_usage_time = last_usage_time
		self.__key_modified['last_usage_time'] = 1

	def get_associated(self):
		"""
		The method to get the associated

		Returns:
			bool: A bool representing the associated
		"""

		return self.__associated

	def set_associated(self, associated):
		"""
		The method to set the value to associated

		Parameters:
			associated (bool) : A bool representing the associated
		"""

		if associated is not None and not isinstance(associated, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: associated EXPECTED TYPE: bool', None, None)
		
		self.__associated = associated
		self.__key_modified['associated'] = 1

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_modified_by(self):
		"""
		The method to get the modified_by

		Returns:
			User: An instance of User
		"""

		return self.__modified_by

	def set_modified_by(self, modified_by):
		"""
		The method to set the value to modified_by

		Parameters:
			modified_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if modified_by is not None and not isinstance(modified_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_by EXPECTED TYPE: User', None, None)
		
		self.__modified_by = modified_by
		self.__key_modified['modified_by'] = 1

	def get_description(self):
		"""
		The method to get the description

		Returns:
			string: A string representing the description
		"""

		return self.__description

	def set_description(self, description):
		"""
		The method to set the value to description

		Parameters:
			description (string) : A string representing the description
		"""

		if description is not None and not isinstance(description, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: description EXPECTED TYPE: str', None, None)
		
		self.__description = description
		self.__key_modified['description'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_editor_mode(self):
		"""
		The method to get the editor_mode

		Returns:
			string: A string representing the editor_mode
		"""

		return self.__editor_mode

	def set_editor_mode(self, editor_mode):
		"""
		The method to set the value to editor_mode

		Parameters:
			editor_mode (string) : A string representing the editor_mode
		"""

		if editor_mode is not None and not isinstance(editor_mode, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: editor_mode EXPECTED TYPE: str', None, None)
		
		self.__editor_mode = editor_mode
		self.__key_modified['editor_mode'] = 1

	def get_favorite(self):
		"""
		The method to get the favorite

		Returns:
			bool: A bool representing the favorite
		"""

		return self.__favorite

	def set_favorite(self, favorite):
		"""
		The method to set the value to favorite

		Parameters:
			favorite (bool) : A bool representing the favorite
		"""

		if favorite is not None and not isinstance(favorite, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: favorite EXPECTED TYPE: bool', None, None)
		
		self.__favorite = favorite
		self.__key_modified['favorite'] = 1

	def get_attachments(self):
		"""
		The method to get the attachments

		Returns:
			list: An instance of list
		"""

		return self.__attachments

	def set_attachments(self, attachments):
		"""
		The method to set the value to attachments

		Parameters:
			attachments (list) : An instance of list
		"""

		if attachments is not None and not isinstance(attachments, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: attachments EXPECTED TYPE: list', None, None)
		
		self.__attachments = attachments
		self.__key_modified['attachments'] = 1

	def get_folder(self):
		"""
		The method to get the folder

		Returns:
			EmailTemplate: An instance of EmailTemplate
		"""

		return self.__folder

	def set_folder(self, folder):
		"""
		The method to set the value to folder

		Parameters:
			folder (EmailTemplate) : An instance of EmailTemplate
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.email_templates.email_template import EmailTemplate
		except Exception:
			from .email_template import EmailTemplate

		if folder is not None and not isinstance(folder, EmailTemplate):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: folder EXPECTED TYPE: EmailTemplate', None, None)
		
		self.__folder = folder
		self.__key_modified['folder'] = 1

	def get_consent_linked(self):
		"""
		The method to get the consent_linked

		Returns:
			bool: A bool representing the consent_linked
		"""

		return self.__consent_linked

	def set_consent_linked(self, consent_linked):
		"""
		The method to set the value to consent_linked

		Parameters:
			consent_linked (bool) : A bool representing the consent_linked
		"""

		if consent_linked is not None and not isinstance(consent_linked, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: consent_linked EXPECTED TYPE: bool', None, None)
		
		self.__consent_linked = consent_linked
		self.__key_modified['consent_linked'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class ValidationError(object):
	def __init__(self):
		"""Creates an instance of ValidationError"""

		self.__api_name = None
		self.__info_message = None
		self.__message = None
		self.__index = None
		self.__parent_api_name = None
		self.__key_modified = dict()

	def get_api_name(self):
		"""
		The method to get the api_name

		Returns:
			string: A string representing the api_name
		"""

		return self.__api_name

	def set_api_name(self, api_name):
		"""
		The method to set the value to api_name

		Parameters:
			api_name (string) : A string representing the api_name
		"""

		if api_name is not None and not isinstance(api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: api_name EXPECTED TYPE: str', None, None)
		
		self.__api_name = api_name
		self.__key_modified['api_name'] = 1

	def get_info_message(self):
		"""
		The method to get the info_message

		Returns:
			string: A string representing the info_message
		"""

		return self.__info_message

	def set_info_message(self, info_message):
		"""
		The method to set the value to info_message

		Parameters:
			info_message (string) : A string representing the info_message
		"""

		if info_message is not None and not isinstance(info_message, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: info_message EXPECTED TYPE: str', None, None)
		
		self.__info_message = info_message
		self.__key_modified['info_message'] = 1

	def get_message(self):
		"""
		The method to get the message

		Returns:
			string: A string representing the message
		"""

		return self.__message

	def set_message(self, message):
		"""
		The method to set the value to message

		Parameters:
			message (string) : A string representing the message
		"""

		if message is not None and not isinstance(message, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: message EXPECTED TYPE: str', None, None)
		
		self.__message = message
		self.__key_modified['message'] = 1

	def get_index(self):
		"""
		The method to get the index

		Returns:
			int: An int representing the index
		"""

		return self.__index

	def set_index(self, index):
		"""
		The method to set the value to index

		Parameters:
			index (int) : An int representing the index
		"""

		if index is not None and not isinstance(index, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: index EXPECTED TYPE: int', None, None)
		
		self.__index = index
		self.__key_modified['index'] = 1

	def get_parent_api_name(self):
		"""
		The method to get the parent_api_name

		Returns:
			string: A string representing the parent_api_name
		"""

		return self.__parent_api_name

	def set_parent_api_name(self, parent_api_name):
		"""
		The method to set the value to parent_api_name

		Parameters:
			parent_api_name (string) : A string representing the parent_api_name
		"""

		if parent_api_name is not None and not isinstance(parent_api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: parent_api_name EXPECTED TYPE: str', None, None)
		
		self.__parent_api_name = parent_api_name
		self.__key_modified['parent_api_name'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class HistoryTracking(object):
	def __init__(self):
		"""Creates an instance of HistoryTracking"""

		self.__module = None
		self.__duration_configured_field = None
		self.__key_modified = dict()

	def get_module(self):
		"""
		The method to get the module

		Returns:
			Module: An instance of Module
		"""

		return self.__module

	def set_module(self, module):
		"""
		The method to set the value to module

		Parameters:
			module (Module) : An instance of Module
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.fields.module import Module
		except Exception:
			from .module import Module

		if module is not None and not isinstance(module, Module):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: module EXPECTED TYPE: Module', None, None)
		
		self.__module = module
		self.__key_modified['module'] = 1

	def get_duration_configured_field(self):
		"""
		The method to get the duration_configured_field

		Returns:
			Field: An instance of Field
		"""

		return self.__duration_configured_field

	def set_duration_configured_field(self, duration_configured_field):
		"""
		The method to set the value to duration_configured_field

		Parameters:
			duration_configured_field (Field) : An instance of Field
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.fields.field import Field
		except Exception:
			from .field import Field

		if duration_configured_field is not None and not isinstance(duration_configured_field, Field):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: duration_configured_field EXPECTED TYPE: Field', None, None)
		
		self.__duration_configured_field = duration_configured_field
		self.__key_modified['duration_configured_field'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class FileDetails(object):
	def __init__(self):
		"""Creates an instance of FileDetails"""

		self.__extn = None
		self.__is_preview_available = None
		self.__download_url = None
		self.__delete_url = None
		self.__entity_id = None
		self.__mode = None
		self.__original_size_byte = None
		self.__preview_url = None
		self.__file_name = None
		self.__file_id = None
		self.__attachment_id = None
		self.__file_size = None
		self.__creator_id = None
		self.__link_docs = None
		self.__delete = None
		self.__key_modified = dict()

	def get_extn(self):
		"""
		The method to get the extn

		Returns:
			string: A string representing the extn
		"""

		return self.__extn

	def set_extn(self, extn):
		"""
		The method to set the value to extn

		Parameters:
			extn (string) : A string representing the extn
		"""

		if extn is not None and not isinstance(extn, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: extn EXPECTED TYPE: str', None, None)
		
		self.__extn = extn
		self.__key_modified['extn'] = 1

	def get_is_preview_available(self):
		"""
		The method to get the is_preview_available

		Returns:
			bool: A bool representing the is_preview_available
		"""

		return self.__is_preview_available

	def set_is_preview_available(self, is_preview_available):
		"""
		The method to set the value to is_preview_available

		Parameters:
			is_preview_available (bool) : A bool representing the is_preview_available
		"""

		if is_preview_available is not None and not isinstance(is_preview_available, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: is_preview_available EXPECTED TYPE: bool', None, None)
		
		self.__is_preview_available = is_preview_available
		self.__key_modified['is_Preview_Available'] = 1

	def get_download_url(self):
		"""
		The method to get the download_url

		Returns:
			string: A string representing the download_url
		"""

		return self.__download_url

	def set_download_url(self, download_url):
		"""
		The method to set the value to download_url

		Parameters:
			download_url (string) : A string representing the download_url
		"""

		if download_url is not None and not isinstance(download_url, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: download_url EXPECTED TYPE: str', None, None)
		
		self.__download_url = download_url
		self.__key_modified['download_Url'] = 1

	def get_delete_url(self):
		"""
		The method to get the delete_url

		Returns:
			string: A string representing the delete_url
		"""

		return self.__delete_url

	def set_delete_url(self, delete_url):
		"""
		The method to set the value to delete_url

		Parameters:
			delete_url (string) : A string representing the delete_url
		"""

		if delete_url is not None and not isinstance(delete_url, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: delete_url EXPECTED TYPE: str', None, None)
		
		self.__delete_url = delete_url
		self.__key_modified['delete_Url'] = 1

	def get_entity_id(self):
		"""
		The method to get the entity_id

		Returns:
			string: A string representing the entity_id
		"""

		return self.__entity_id

	def set_entity_id(self, entity_id):
		"""
		The method to set the value to entity_id

		Parameters:
			entity_id (string) : A string representing the entity_id
		"""

		if entity_id is not None and not isinstance(entity_id, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: entity_id EXPECTED TYPE: str', None, None)
		
		self.__entity_id = entity_id
		self.__key_modified['entity_Id'] = 1

	def get_mode(self):
		"""
		The method to get the mode

		Returns:
			string: A string representing the mode
		"""

		return self.__mode

	def set_mode(self, mode):
		"""
		The method to set the value to mode

		Parameters:
			mode (string) : A string representing the mode
		"""

		if mode is not None and not isinstance(mode, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: mode EXPECTED TYPE: str', None, None)
		
		self.__mode = mode
		self.__key_modified['mode'] = 1

	def get_original_size_byte(self):
		"""
		The method to get the original_size_byte

		Returns:
			string: A string representing the original_size_byte
		"""

		return self.__original_size_byte

	def set_original_size_byte(self, original_size_byte):
		"""
		The method to set the value to original_size_byte

		Parameters:
			original_size_byte (string) : A string representing the original_size_byte
		"""

		if original_size_byte is not None and not isinstance(original_size_byte, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: original_size_byte EXPECTED TYPE: str', None, None)
		
		self.__original_size_byte = original_size_byte
		self.__key_modified['original_Size_Byte'] = 1

	def get_preview_url(self):
		"""
		The method to get the preview_url

		Returns:
			string: A string representing the preview_url
		"""

		return self.__preview_url

	def set_preview_url(self, preview_url):
		"""
		The method to set the value to preview_url

		Parameters:
			preview_url (string) : A string representing the preview_url
		"""

		if preview_url is not None and not isinstance(preview_url, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: preview_url EXPECTED TYPE: str', None, None)
		
		self.__preview_url = preview_url
		self.__key_modified['preview_Url'] = 1

	def get_file_name(self):
		"""
		The method to get the file_name

		Returns:
			string: A string representing the file_name
		"""

		return self.__file_name

	def set_file_name(self, file_name):
		"""
		The method to set the value to file_name

		Parameters:
			file_name (string) : A string representing the file_name
		"""

		if file_name is not None and not isinstance(file_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: file_name EXPECTED TYPE: str', None, None)
		
		self.__file_name = file_name
		self.__key_modified['file_Name'] = 1

	def get_file_id(self):
		"""
		The method to get the file_id

		Returns:
			string: A string representing the file_id
		"""

		return self.__file_id

	def set_file_id(self, file_id):
		"""
		The method to set the value to file_id

		Parameters:
			file_id (string) : A string representing the file_id
		"""

		if file_id is not None and not isinstance(file_id, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: file_id EXPECTED TYPE: str', None, None)
		
		self.__file_id = file_id
		self.__key_modified['file_Id'] = 1

	def get_attachment_id(self):
		"""
		The method to get the attachment_id

		Returns:
			string: A string representing the attachment_id
		"""

		return self.__attachment_id

	def set_attachment_id(self, attachment_id):
		"""
		The method to set the value to attachment_id

		Parameters:
			attachment_id (string) : A string representing the attachment_id
		"""

		if attachment_id is not None and not isinstance(attachment_id, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: attachment_id EXPECTED TYPE: str', None, None)
		
		self.__attachment_id = attachment_id
		self.__key_modified['attachment_Id'] = 1

	def get_file_size(self):
		"""
		The method to get the file_size

		Returns:
			string: A string representing the file_size
		"""

		return self.__file_size

	def set_file_size(self, file_size):
		"""
		The method to set the value to file_size

		Parameters:
			file_size (string) : A string representing the file_size
		"""

		if file_size is not None and not isinstance(file_size, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: file_size EXPECTED TYPE: str', None, None)
		
		self.__file_size = file_size
		self.__key_modified['file_Size'] = 1

	def get_creator_id(self):
		"""
		The method to get the creator_id

		Returns:
			string: A string representing the creator_id
		"""

		return self.__creator_id

	def set_creator_id(self, creator_id):
		"""
		The method to set the value to creator_id

		Parameters:
			creator_id (string) : A string representing the creator_id
		"""

		if creator_id is not None and not isinstance(creator_id, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: creator_id EXPECTED TYPE: str', None, None)
		
		self.__creator_id = creator_id
		self.__key_modified['creator_Id'] = 1

	def get_link_docs(self):
		"""
		The method to get the link_docs

		Returns:
			int: An int representing the link_docs
		"""

		return self.__link_docs

	def set_link_docs(self, link_docs):
		"""
		The method to set the value to link_docs

		Parameters:
			link_docs (int) : An int representing the link_docs
		"""

		if link_docs is not None and not isinstance(link_docs, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: link_docs EXPECTED TYPE: int', None, None)
		
		self.__link_docs = link_docs
		self.__key_modified['link_Docs'] = 1

	def get_delete(self):
		"""
		The method to get the delete

		Returns:
			string: A string representing the delete
		"""

		return self.__delete

	def set_delete(self, delete):
		"""
		The method to set the value to delete

		Parameters:
			delete (string) : A string representing the delete
		"""

		if delete is not None and not isinstance(delete, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: delete EXPECTED TYPE: str', None, None)
		
		self.__delete = delete
		self.__key_modified['_delete'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Choice, Constants
	from zcrmsdk.src.com.zoho.crm.api.bulk_write.action_response import ActionResponse
	from zcrmsdk.src.com.zoho.crm.api.bulk_write.response_handler import ResponseHandler
	from zcrmsdk.src.com.zoho.crm.api.bulk_write.response_wrapper import ResponseWrapper
except Exception:
	from ..exception import SDKException
	from ..util import Choice, Constants
	from .action_response import ActionResponse
	from .response_handler import ResponseHandler
	from .response_wrapper import ResponseWrapper


class APIException(ActionResponse, ResponseWrapper, ResponseHandler):
	def __init__(self):
		"""Creates an instance of APIException"""
		super().__init__()

		self.__code = None
		self.__message = None
		self.__status = None
		self.__details = None
		self.__error_message = None
		self.__error_code = None
		self.__x_error = None
		self.__info = None
		self.__x_info = None
		self.__http_status = None
		self.__key_modified = dict()

	def get_code(self):
		"""
		The method to get the code

		Returns:
			Choice: An instance of Choice
		"""

		return self.__code

	def set_code(self, code):
		"""
		The method to set the value to code

		Parameters:
			code (Choice) : An instance of Choice
		"""

		if code is not None and not isinstance(code, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: code EXPECTED TYPE: Choice', None, None)
		
		self.__code = code
		self.__key_modified['code'] = 1

	def get_message(self):
		"""
		The method to get the message

		Returns:
			Choice: An instance of Choice
		"""

		return self.__message

	def set_message(self, message):
		"""
		The method to set the value to message

		Parameters:
			message (Choice) : An instance of Choice
		"""

		if message is not None and not isinstance(message, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: message EXPECTED TYPE: Choice', None, None)
		
		self.__message = message
		self.__key_modified['message'] = 1

	def get_status(self):
		"""
		The method to get the status

		Returns:
			Choice: An instance of Choice
		"""

		return self.__status

	def set_status(self, status):
		"""
		The method to set the value to status

		Parameters:
			status (Choice) : An instance of Choice
		"""

		if status is not None and not isinstance(status, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: status EXPECTED TYPE: Choice', None, None)
		
		self.__status = status
		self.__key_modified['status'] = 1

	def get_details(self):
		"""
		The method to get the details

		Returns:
			dict: An instance of dict
		"""

		return self.__details

	def set_details(self, details):
		"""
		The method to set the value to details

		Parameters:
			details (dict) : An instance of dict
		"""

		if details is not None and not isinstance(details, dict):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: details EXPECTED TYPE: dict', None, None)
		
		self.__details = details
		self.__key_modified['details'] = 1

	def get_error_message(self):
		"""
		The method to get the error_message

		Returns:
			Choice: An instance of Choice
		"""

		return self.__error_message

	def set_error_message(self, error_message):
		"""
		The method to set the value to error_message

		Parameters:
			error_message (Choice) : An instance of Choice
		"""

		if error_message is not None and not isinstance(error_message, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: error_message EXPECTED TYPE: Choice', None, None)
		
		self.__error_message = error_message
		self.__key_modified['ERROR_MESSAGE'] = 1

	def get_error_code(self):
		"""
		The method to get the error_code

		Returns:
			int: An int representing the error_code
		"""

		return self.__error_code

	def set_error_code(self, error_code):
		"""
		The method to set the value to error_code

		Parameters:
			error_code (int) : An int representing the error_code
		"""

		if error_code is not None and not isinstance(error_code, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: error_code EXPECTED TYPE: int', None, None)
		
		self.__error_code = error_code
		self.__key_modified['ERROR_CODE'] = 1

	def get_x_error(self):
		"""
		The method to get the x_error

		Returns:
			Choice: An instance of Choice
		"""

		return self.__x_error

	def set_x_error(self, x_error):
		"""
		The method to set the value to x_error

		Parameters:
			x_error (Choice) : An instance of Choice
		"""

		if x_error is not None and not isinstance(x_error, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: x_error EXPECTED TYPE: Choice', None, None)
		
		self.__x_error = x_error
		self.__key_modified['x-error'] = 1

	def get_info(self):
		"""
		The method to get the info

		Returns:
			Choice: An instance of Choice
		"""

		return self.__info

	def set_info(self, info):
		"""
		The method to set the value to info

		Parameters:
			info (Choice) : An instance of Choice
		"""

		if info is not None and not isinstance(info, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: info EXPECTED TYPE: Choice', None, None)
		
		self.__info = info
		self.__key_modified['info'] = 1

	def get_x_info(self):
		"""
		The method to get the x_info

		Returns:
			Choice: An instance of Choice
		"""

		return self.__x_info

	def set_x_info(self, x_info):
		"""
		The method to set the value to x_info

		Parameters:
			x_info (Choice) : An instance of Choice
		"""

		if x_info is not None and not isinstance(x_info, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: x_info EXPECTED TYPE: Choice', None, None)
		
		self.__x_info = x_info
		self.__key_modified['x-info'] = 1

	def get_http_status(self):
		"""
		The method to get the http_status

		Returns:
			string: A string representing the http_status
		"""

		return self.__http_status

	def set_http_status(self, http_status):
		"""
		The method to set the value to http_status

		Parameters:
			http_status (string) : A string representing the http_status
		"""

		if http_status is not None and not isinstance(http_status, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: http_status EXPECTED TYPE: str', None, None)
		
		self.__http_status = http_status
		self.__key_modified['http_status'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



class SDKConfig(object):
    """
    The class to configure the SDK.
    """

    def __init__(self, auto_refresh_fields=False, pick_list_validation=True, read_timeout=None, connect_timeout=None):
        """
        Creates an instance of SDKConfig with the following parameters

        Parameters:
            auto_refresh_fields(bool): A bool representing auto_refresh_fields
            pick_list_validation(bool): A bool representing pick_list_validation
            read_timeout(float): A Float representing read_timeout
            connect_timeout(float): A Float representing connect_timeout
        """
        self.__auto_refresh_fields = auto_refresh_fields
        self.__pick_list_validation = pick_list_validation
        self.__read_timeout = read_timeout
        self.__connect_timeout = connect_timeout

    def get_auto_refresh_fields(self):
        """
        This is a getter method to get auto_refresh_fields.

        Returns:
            bool: A bool representing auto_refresh_fields
        """
        return self.__auto_refresh_fields

    def get_pick_list_validation(self):
        """
        This is a getter method to get pick_list_validation.

        Returns:
            bool: A bool representing pick_list_validation
        """
        return self.__pick_list_validation

    def get_read_timeout(self):
        """
        This is a getter method to get read_timeout.

        Returns:
            Float: A Float representing read_timeout
        """
        return self.__read_timeout

    def get_connect_timeout(self):
        """
        This is a getter method to get connect_timeout.

        Returns:
            Float: A Float representing connect_timeout
        """
        return self.__connect_timeout



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Wizard(object):
	def __init__(self):
		"""Creates an instance of Wizard"""

		self.__created_time = None
		self.__modified_time = None
		self.__module = None
		self.__name = None
		self.__modified_by = None
		self.__profiles = None
		self.__active = None
		self.__containers = None
		self.__created_by = None
		self.__parent_wizard = None
		self.__draft = None
		self.__id = None
		self.__key_modified = dict()

	def get_created_time(self):
		"""
		The method to get the created_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__created_time

	def set_created_time(self, created_time):
		"""
		The method to set the value to created_time

		Parameters:
			created_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if created_time is not None and not isinstance(created_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_time EXPECTED TYPE: datetime', None, None)
		
		self.__created_time = created_time
		self.__key_modified['created_time'] = 1

	def get_modified_time(self):
		"""
		The method to get the modified_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__modified_time

	def set_modified_time(self, modified_time):
		"""
		The method to set the value to modified_time

		Parameters:
			modified_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if modified_time is not None and not isinstance(modified_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_time EXPECTED TYPE: datetime', None, None)
		
		self.__modified_time = modified_time
		self.__key_modified['modified_time'] = 1

	def get_module(self):
		"""
		The method to get the module

		Returns:
			Module: An instance of Module
		"""

		return self.__module

	def set_module(self, module):
		"""
		The method to set the value to module

		Parameters:
			module (Module) : An instance of Module
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.modules import Module
		except Exception:
			from ..modules import Module

		if module is not None and not isinstance(module, Module):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: module EXPECTED TYPE: Module', None, None)
		
		self.__module = module
		self.__key_modified['module'] = 1

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_modified_by(self):
		"""
		The method to get the modified_by

		Returns:
			User: An instance of User
		"""

		return self.__modified_by

	def set_modified_by(self, modified_by):
		"""
		The method to set the value to modified_by

		Parameters:
			modified_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if modified_by is not None and not isinstance(modified_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_by EXPECTED TYPE: User', None, None)
		
		self.__modified_by = modified_by
		self.__key_modified['modified_by'] = 1

	def get_profiles(self):
		"""
		The method to get the profiles

		Returns:
			list: An instance of list
		"""

		return self.__profiles

	def set_profiles(self, profiles):
		"""
		The method to set the value to profiles

		Parameters:
			profiles (list) : An instance of list
		"""

		if profiles is not None and not isinstance(profiles, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: profiles EXPECTED TYPE: list', None, None)
		
		self.__profiles = profiles
		self.__key_modified['profiles'] = 1

	def get_active(self):
		"""
		The method to get the active

		Returns:
			bool: A bool representing the active
		"""

		return self.__active

	def set_active(self, active):
		"""
		The method to set the value to active

		Parameters:
			active (bool) : A bool representing the active
		"""

		if active is not None and not isinstance(active, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: active EXPECTED TYPE: bool', None, None)
		
		self.__active = active
		self.__key_modified['active'] = 1

	def get_containers(self):
		"""
		The method to get the containers

		Returns:
			list: An instance of list
		"""

		return self.__containers

	def set_containers(self, containers):
		"""
		The method to set the value to containers

		Parameters:
			containers (list) : An instance of list
		"""

		if containers is not None and not isinstance(containers, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: containers EXPECTED TYPE: list', None, None)
		
		self.__containers = containers
		self.__key_modified['containers'] = 1

	def get_created_by(self):
		"""
		The method to get the created_by

		Returns:
			User: An instance of User
		"""

		return self.__created_by

	def set_created_by(self, created_by):
		"""
		The method to set the value to created_by

		Parameters:
			created_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if created_by is not None and not isinstance(created_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by EXPECTED TYPE: User', None, None)
		
		self.__created_by = created_by
		self.__key_modified['created_by'] = 1

	def get_parent_wizard(self):
		"""
		The method to get the parent_wizard

		Returns:
			Wizard: An instance of Wizard
		"""

		return self.__parent_wizard

	def set_parent_wizard(self, parent_wizard):
		"""
		The method to set the value to parent_wizard

		Parameters:
			parent_wizard (Wizard) : An instance of Wizard
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.wizards.wizard import Wizard
		except Exception:
			from .wizard import Wizard

		if parent_wizard is not None and not isinstance(parent_wizard, Wizard):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: parent_wizard EXPECTED TYPE: Wizard', None, None)
		
		self.__parent_wizard = parent_wizard
		self.__key_modified['parent_wizard'] = 1

	def get_draft(self):
		"""
		The method to get the draft

		Returns:
			bool: A bool representing the draft
		"""

		return self.__draft

	def set_draft(self, draft):
		"""
		The method to set the value to draft

		Parameters:
			draft (bool) : A bool representing the draft
		"""

		if draft is not None and not isinstance(draft, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: draft EXPECTED TYPE: bool', None, None)
		
		self.__draft = draft
		self.__key_modified['draft'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



import json


class SDKException(Exception):

    """
    This class is the common SDKException object.
    """

    message = 'Caused By: {code} - {message}'

    def __init__(self, code=None, message=None, details=None, cause=None):

        """
        Creates an SDKException class instance with the specified parameters.

        Parameters:
            code (str) : A string containing the Exception error code.
            message (str) : A string containing the Exception error message.
            details (dict) : A dict containing the error response.
            cause (Exception) : A Exception class instance.
        """

        self.code = code
        self.cause = cause
        self.details = details
        self.error_message = "" if message is None else message

        if self.details is not None:
            self.error_message = self.error_message + json.dumps(self.details)

        if self.cause is not None:
            self.error_message = self.error_message + str(self.cause)

        Exception.__init__(self, code, message)

    def __str__(self):
        return self.message.format(code=self.code, message=self.error_message)



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class InventoryDetails(object):
	def __init__(self):
		"""Creates an instance of InventoryDetails"""

		self.__inventory_template = None
		self.__paper_type = None
		self.__view_type = None
		self.__key_modified = dict()

	def get_inventory_template(self):
		"""
		The method to get the inventory_template

		Returns:
			InventoryTemplate: An instance of InventoryTemplate
		"""

		return self.__inventory_template

	def set_inventory_template(self, inventory_template):
		"""
		The method to set the value to inventory_template

		Parameters:
			inventory_template (InventoryTemplate) : An instance of InventoryTemplate
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.inventorytemplates import InventoryTemplate
		except Exception:
			from ..inventory_templates import InventoryTemplate

		if inventory_template is not None and not isinstance(inventory_template, InventoryTemplate):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: inventory_template EXPECTED TYPE: InventoryTemplate', None, None)
		
		self.__inventory_template = inventory_template
		self.__key_modified['inventory_template'] = 1

	def get_paper_type(self):
		"""
		The method to get the paper_type

		Returns:
			string: A string representing the paper_type
		"""

		return self.__paper_type

	def set_paper_type(self, paper_type):
		"""
		The method to set the value to paper_type

		Parameters:
			paper_type (string) : A string representing the paper_type
		"""

		if paper_type is not None and not isinstance(paper_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: paper_type EXPECTED TYPE: str', None, None)
		
		self.__paper_type = paper_type
		self.__key_modified['paper_type'] = 1

	def get_view_type(self):
		"""
		The method to get the view_type

		Returns:
			string: A string representing the view_type
		"""

		return self.__view_type

	def set_view_type(self, view_type):
		"""
		The method to set the value to view_type

		Parameters:
			view_type (string) : A string representing the view_type
		"""

		if view_type is not None and not isinstance(view_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: view_type EXPECTED TYPE: str', None, None)
		
		self.__view_type = view_type
		self.__key_modified['view_type'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Note(object):
	def __init__(self):
		"""Creates an instance of Note"""

		self.__owner = None
		self.__modified_time = None
		self.__attachments = None
		self.__created_time = None
		self.__parent_id = None
		self.__editable = None
		self.__se_module = None
		self.__is_shared_to_client = None
		self.__modified_by = None
		self.__size = None
		self.__state = None
		self.__voice_note = None
		self.__id = None
		self.__created_by = None
		self.__note_title = None
		self.__note_content = None
		self.__key_modified = dict()

	def get_owner(self):
		"""
		The method to get the owner

		Returns:
			User: An instance of User
		"""

		return self.__owner

	def set_owner(self, owner):
		"""
		The method to set the value to owner

		Parameters:
			owner (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if owner is not None and not isinstance(owner, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: owner EXPECTED TYPE: User', None, None)
		
		self.__owner = owner
		self.__key_modified['Owner'] = 1

	def get_modified_time(self):
		"""
		The method to get the modified_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__modified_time

	def set_modified_time(self, modified_time):
		"""
		The method to set the value to modified_time

		Parameters:
			modified_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if modified_time is not None and not isinstance(modified_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_time EXPECTED TYPE: datetime', None, None)
		
		self.__modified_time = modified_time
		self.__key_modified['Modified_Time'] = 1

	def get_attachments(self):
		"""
		The method to get the attachments

		Returns:
			list: An instance of list
		"""

		return self.__attachments

	def set_attachments(self, attachments):
		"""
		The method to set the value to attachments

		Parameters:
			attachments (list) : An instance of list
		"""

		if attachments is not None and not isinstance(attachments, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: attachments EXPECTED TYPE: list', None, None)
		
		self.__attachments = attachments
		self.__key_modified['$attachments'] = 1

	def get_created_time(self):
		"""
		The method to get the created_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__created_time

	def set_created_time(self, created_time):
		"""
		The method to set the value to created_time

		Parameters:
			created_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if created_time is not None and not isinstance(created_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_time EXPECTED TYPE: datetime', None, None)
		
		self.__created_time = created_time
		self.__key_modified['Created_Time'] = 1

	def get_parent_id(self):
		"""
		The method to get the parent_id

		Returns:
			Record: An instance of Record
		"""

		return self.__parent_id

	def set_parent_id(self, parent_id):
		"""
		The method to set the value to parent_id

		Parameters:
			parent_id (Record) : An instance of Record
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.record import Record
		except Exception:
			from ..record import Record

		if parent_id is not None and not isinstance(parent_id, Record):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: parent_id EXPECTED TYPE: Record', None, None)
		
		self.__parent_id = parent_id
		self.__key_modified['Parent_Id'] = 1

	def get_editable(self):
		"""
		The method to get the editable

		Returns:
			bool: A bool representing the editable
		"""

		return self.__editable

	def set_editable(self, editable):
		"""
		The method to set the value to editable

		Parameters:
			editable (bool) : A bool representing the editable
		"""

		if editable is not None and not isinstance(editable, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: editable EXPECTED TYPE: bool', None, None)
		
		self.__editable = editable
		self.__key_modified['$editable'] = 1

	def get_se_module(self):
		"""
		The method to get the se_module

		Returns:
			string: A string representing the se_module
		"""

		return self.__se_module

	def set_se_module(self, se_module):
		"""
		The method to set the value to se_module

		Parameters:
			se_module (string) : A string representing the se_module
		"""

		if se_module is not None and not isinstance(se_module, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: se_module EXPECTED TYPE: str', None, None)
		
		self.__se_module = se_module
		self.__key_modified['$se_module'] = 1

	def get_is_shared_to_client(self):
		"""
		The method to get the is_shared_to_client

		Returns:
			bool: A bool representing the is_shared_to_client
		"""

		return self.__is_shared_to_client

	def set_is_shared_to_client(self, is_shared_to_client):
		"""
		The method to set the value to is_shared_to_client

		Parameters:
			is_shared_to_client (bool) : A bool representing the is_shared_to_client
		"""

		if is_shared_to_client is not None and not isinstance(is_shared_to_client, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: is_shared_to_client EXPECTED TYPE: bool', None, None)
		
		self.__is_shared_to_client = is_shared_to_client
		self.__key_modified['$is_shared_to_client'] = 1

	def get_modified_by(self):
		"""
		The method to get the modified_by

		Returns:
			User: An instance of User
		"""

		return self.__modified_by

	def set_modified_by(self, modified_by):
		"""
		The method to set the value to modified_by

		Parameters:
			modified_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if modified_by is not None and not isinstance(modified_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_by EXPECTED TYPE: User', None, None)
		
		self.__modified_by = modified_by
		self.__key_modified['Modified_By'] = 1

	def get_size(self):
		"""
		The method to get the size

		Returns:
			string: A string representing the size
		"""

		return self.__size

	def set_size(self, size):
		"""
		The method to set the value to size

		Parameters:
			size (string) : A string representing the size
		"""

		if size is not None and not isinstance(size, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: size EXPECTED TYPE: str', None, None)
		
		self.__size = size
		self.__key_modified['$size'] = 1

	def get_state(self):
		"""
		The method to get the state

		Returns:
			string: A string representing the state
		"""

		return self.__state

	def set_state(self, state):
		"""
		The method to set the value to state

		Parameters:
			state (string) : A string representing the state
		"""

		if state is not None and not isinstance(state, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: state EXPECTED TYPE: str', None, None)
		
		self.__state = state
		self.__key_modified['$state'] = 1

	def get_voice_note(self):
		"""
		The method to get the voice_note

		Returns:
			bool: A bool representing the voice_note
		"""

		return self.__voice_note

	def set_voice_note(self, voice_note):
		"""
		The method to set the value to voice_note

		Parameters:
			voice_note (bool) : A bool representing the voice_note
		"""

		if voice_note is not None and not isinstance(voice_note, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: voice_note EXPECTED TYPE: bool', None, None)
		
		self.__voice_note = voice_note
		self.__key_modified['$voice_note'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_created_by(self):
		"""
		The method to get the created_by

		Returns:
			User: An instance of User
		"""

		return self.__created_by

	def set_created_by(self, created_by):
		"""
		The method to set the value to created_by

		Parameters:
			created_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if created_by is not None and not isinstance(created_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by EXPECTED TYPE: User', None, None)
		
		self.__created_by = created_by
		self.__key_modified['Created_By'] = 1

	def get_note_title(self):
		"""
		The method to get the note_title

		Returns:
			string: A string representing the note_title
		"""

		return self.__note_title

	def set_note_title(self, note_title):
		"""
		The method to set the value to note_title

		Parameters:
			note_title (string) : A string representing the note_title
		"""

		if note_title is not None and not isinstance(note_title, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: note_title EXPECTED TYPE: str', None, None)
		
		self.__note_title = note_title
		self.__key_modified['Note_Title'] = 1

	def get_note_content(self):
		"""
		The method to get the note_content

		Returns:
			string: A string representing the note_content
		"""

		return self.__note_content

	def set_note_content(self, note_content):
		"""
		The method to set the value to note_content

		Parameters:
			note_content (string) : A string representing the note_content
		"""

		if note_content is not None and not isinstance(note_content, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: note_content EXPECTED TYPE: str', None, None)
		
		self.__note_content = note_content
		self.__key_modified['Note_Content'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class ProcessInfo(object):
	def __init__(self):
		"""Creates an instance of ProcessInfo"""

		self.__field_id = None
		self.__is_continuous = None
		self.__api_name = None
		self.__continuous = None
		self.__field_label = None
		self.__name = None
		self.__column_name = None
		self.__field_value = None
		self.__id = None
		self.__field_name = None
		self.__escalation = None
		self.__key_modified = dict()

	def get_field_id(self):
		"""
		The method to get the field_id

		Returns:
			string: A string representing the field_id
		"""

		return self.__field_id

	def set_field_id(self, field_id):
		"""
		The method to set the value to field_id

		Parameters:
			field_id (string) : A string representing the field_id
		"""

		if field_id is not None and not isinstance(field_id, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: field_id EXPECTED TYPE: str', None, None)
		
		self.__field_id = field_id
		self.__key_modified['field_id'] = 1

	def get_is_continuous(self):
		"""
		The method to get the is_continuous

		Returns:
			bool: A bool representing the is_continuous
		"""

		return self.__is_continuous

	def set_is_continuous(self, is_continuous):
		"""
		The method to set the value to is_continuous

		Parameters:
			is_continuous (bool) : A bool representing the is_continuous
		"""

		if is_continuous is not None and not isinstance(is_continuous, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: is_continuous EXPECTED TYPE: bool', None, None)
		
		self.__is_continuous = is_continuous
		self.__key_modified['is_continuous'] = 1

	def get_api_name(self):
		"""
		The method to get the api_name

		Returns:
			string: A string representing the api_name
		"""

		return self.__api_name

	def set_api_name(self, api_name):
		"""
		The method to set the value to api_name

		Parameters:
			api_name (string) : A string representing the api_name
		"""

		if api_name is not None and not isinstance(api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: api_name EXPECTED TYPE: str', None, None)
		
		self.__api_name = api_name
		self.__key_modified['api_name'] = 1

	def get_continuous(self):
		"""
		The method to get the continuous

		Returns:
			bool: A bool representing the continuous
		"""

		return self.__continuous

	def set_continuous(self, continuous):
		"""
		The method to set the value to continuous

		Parameters:
			continuous (bool) : A bool representing the continuous
		"""

		if continuous is not None and not isinstance(continuous, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: continuous EXPECTED TYPE: bool', None, None)
		
		self.__continuous = continuous
		self.__key_modified['continuous'] = 1

	def get_field_label(self):
		"""
		The method to get the field_label

		Returns:
			string: A string representing the field_label
		"""

		return self.__field_label

	def set_field_label(self, field_label):
		"""
		The method to set the value to field_label

		Parameters:
			field_label (string) : A string representing the field_label
		"""

		if field_label is not None and not isinstance(field_label, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: field_label EXPECTED TYPE: str', None, None)
		
		self.__field_label = field_label
		self.__key_modified['field_label'] = 1

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_column_name(self):
		"""
		The method to get the column_name

		Returns:
			string: A string representing the column_name
		"""

		return self.__column_name

	def set_column_name(self, column_name):
		"""
		The method to set the value to column_name

		Parameters:
			column_name (string) : A string representing the column_name
		"""

		if column_name is not None and not isinstance(column_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: column_name EXPECTED TYPE: str', None, None)
		
		self.__column_name = column_name
		self.__key_modified['column_name'] = 1

	def get_field_value(self):
		"""
		The method to get the field_value

		Returns:
			string: A string representing the field_value
		"""

		return self.__field_value

	def set_field_value(self, field_value):
		"""
		The method to set the value to field_value

		Parameters:
			field_value (string) : A string representing the field_value
		"""

		if field_value is not None and not isinstance(field_value, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: field_value EXPECTED TYPE: str', None, None)
		
		self.__field_value = field_value
		self.__key_modified['field_value'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_field_name(self):
		"""
		The method to get the field_name

		Returns:
			string: A string representing the field_name
		"""

		return self.__field_name

	def set_field_name(self, field_name):
		"""
		The method to set the value to field_name

		Parameters:
			field_name (string) : A string representing the field_name
		"""

		if field_name is not None and not isinstance(field_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: field_name EXPECTED TYPE: str', None, None)
		
		self.__field_name = field_name
		self.__key_modified['field_name'] = 1

	def get_escalation(self):
		"""
		The method to get the escalation

		Returns:
			Escalation: An instance of Escalation
		"""

		return self.__escalation

	def set_escalation(self, escalation):
		"""
		The method to set the value to escalation

		Parameters:
			escalation (Escalation) : An instance of Escalation
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.blue_print.escalation import Escalation
		except Exception:
			from .escalation import Escalation

		if escalation is not None and not isinstance(escalation, Escalation):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: escalation EXPECTED TYPE: Escalation', None, None)
		
		self.__escalation = escalation
		self.__key_modified['escalation'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class LicenseDetails(object):
	def __init__(self):
		"""Creates an instance of LicenseDetails"""

		self.__paid_expiry = None
		self.__users_license_purchased = None
		self.__trial_type = None
		self.__trial_expiry = None
		self.__paid = None
		self.__paid_type = None
		self.__key_modified = dict()

	def get_paid_expiry(self):
		"""
		The method to get the paid_expiry

		Returns:
			datetime: An instance of datetime
		"""

		return self.__paid_expiry

	def set_paid_expiry(self, paid_expiry):
		"""
		The method to set the value to paid_expiry

		Parameters:
			paid_expiry (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if paid_expiry is not None and not isinstance(paid_expiry, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: paid_expiry EXPECTED TYPE: datetime', None, None)
		
		self.__paid_expiry = paid_expiry
		self.__key_modified['paid_expiry'] = 1

	def get_users_license_purchased(self):
		"""
		The method to get the users_license_purchased

		Returns:
			int: An int representing the users_license_purchased
		"""

		return self.__users_license_purchased

	def set_users_license_purchased(self, users_license_purchased):
		"""
		The method to set the value to users_license_purchased

		Parameters:
			users_license_purchased (int) : An int representing the users_license_purchased
		"""

		if users_license_purchased is not None and not isinstance(users_license_purchased, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: users_license_purchased EXPECTED TYPE: int', None, None)
		
		self.__users_license_purchased = users_license_purchased
		self.__key_modified['users_license_purchased'] = 1

	def get_trial_type(self):
		"""
		The method to get the trial_type

		Returns:
			string: A string representing the trial_type
		"""

		return self.__trial_type

	def set_trial_type(self, trial_type):
		"""
		The method to set the value to trial_type

		Parameters:
			trial_type (string) : A string representing the trial_type
		"""

		if trial_type is not None and not isinstance(trial_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: trial_type EXPECTED TYPE: str', None, None)
		
		self.__trial_type = trial_type
		self.__key_modified['trial_type'] = 1

	def get_trial_expiry(self):
		"""
		The method to get the trial_expiry

		Returns:
			string: A string representing the trial_expiry
		"""

		return self.__trial_expiry

	def set_trial_expiry(self, trial_expiry):
		"""
		The method to set the value to trial_expiry

		Parameters:
			trial_expiry (string) : A string representing the trial_expiry
		"""

		if trial_expiry is not None and not isinstance(trial_expiry, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: trial_expiry EXPECTED TYPE: str', None, None)
		
		self.__trial_expiry = trial_expiry
		self.__key_modified['trial_expiry'] = 1

	def get_paid(self):
		"""
		The method to get the paid

		Returns:
			bool: A bool representing the paid
		"""

		return self.__paid

	def set_paid(self, paid):
		"""
		The method to set the value to paid

		Parameters:
			paid (bool) : A bool representing the paid
		"""

		if paid is not None and not isinstance(paid, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: paid EXPECTED TYPE: bool', None, None)
		
		self.__paid = paid
		self.__key_modified['paid'] = 1

	def get_paid_type(self):
		"""
		The method to get the paid_type

		Returns:
			string: A string representing the paid_type
		"""

		return self.__paid_type

	def set_paid_type(self, paid_type):
		"""
		The method to set the value to paid_type

		Parameters:
			paid_type (string) : A string representing the paid_type
		"""

		if paid_type is not None and not isinstance(paid_type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: paid_type EXPECTED TYPE: str', None, None)
		
		self.__paid_type = paid_type
		self.__key_modified['paid_type'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Variable(object):
	def __init__(self):
		"""Creates an instance of Variable"""

		self.__api_name = None
		self.__name = None
		self.__description = None
		self.__id = None
		self.__type = None
		self.__variable_group = None
		self.__value = None
		self.__key_modified = dict()

	def get_api_name(self):
		"""
		The method to get the api_name

		Returns:
			string: A string representing the api_name
		"""

		return self.__api_name

	def set_api_name(self, api_name):
		"""
		The method to set the value to api_name

		Parameters:
			api_name (string) : A string representing the api_name
		"""

		if api_name is not None and not isinstance(api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: api_name EXPECTED TYPE: str', None, None)
		
		self.__api_name = api_name
		self.__key_modified['api_name'] = 1

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_description(self):
		"""
		The method to get the description

		Returns:
			string: A string representing the description
		"""

		return self.__description

	def set_description(self, description):
		"""
		The method to set the value to description

		Parameters:
			description (string) : A string representing the description
		"""

		if description is not None and not isinstance(description, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: description EXPECTED TYPE: str', None, None)
		
		self.__description = description
		self.__key_modified['description'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_type(self):
		"""
		The method to get the type

		Returns:
			string: A string representing the type
		"""

		return self.__type

	def set_type(self, type):
		"""
		The method to set the value to type

		Parameters:
			type (string) : A string representing the type
		"""

		if type is not None and not isinstance(type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: type EXPECTED TYPE: str', None, None)
		
		self.__type = type
		self.__key_modified['type'] = 1

	def get_variable_group(self):
		"""
		The method to get the variable_group

		Returns:
			VariableGroup: An instance of VariableGroup
		"""

		return self.__variable_group

	def set_variable_group(self, variable_group):
		"""
		The method to set the value to variable_group

		Parameters:
			variable_group (VariableGroup) : An instance of VariableGroup
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.variablegroups import VariableGroup
		except Exception:
			from ..variable_groups import VariableGroup

		if variable_group is not None and not isinstance(variable_group, VariableGroup):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: variable_group EXPECTED TYPE: VariableGroup', None, None)
		
		self.__variable_group = variable_group
		self.__key_modified['variable_group'] = 1

	def get_value(self):
		"""
		The method to get the value

		Returns:
			Object: A Object representing the value
		"""

		return self.__value

	def set_value(self, value):
		"""
		The method to set the value to value

		Parameters:
			value (Object) : A Object representing the value
		"""

		self.__value = value
		self.__key_modified['value'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Choice, Constants
	from zcrmsdk.src.com.zoho.crm.api.attachments.action_response import ActionResponse
	from zcrmsdk.src.com.zoho.crm.api.attachments.response_handler import ResponseHandler
	from zcrmsdk.src.com.zoho.crm.api.attachments.action_handler import ActionHandler
except Exception:
	from ..exception import SDKException
	from ..util import Choice, Constants
	from .action_response import ActionResponse
	from .response_handler import ResponseHandler
	from .action_handler import ActionHandler


class APIException(ResponseHandler, ActionResponse, ActionHandler):
	def __init__(self):
		"""Creates an instance of APIException"""
		super().__init__()

		self.__status = None
		self.__code = None
		self.__message = None
		self.__details = None
		self.__key_modified = dict()

	def get_status(self):
		"""
		The method to get the status

		Returns:
			Choice: An instance of Choice
		"""

		return self.__status

	def set_status(self, status):
		"""
		The method to set the value to status

		Parameters:
			status (Choice) : An instance of Choice
		"""

		if status is not None and not isinstance(status, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: status EXPECTED TYPE: Choice', None, None)
		
		self.__status = status
		self.__key_modified['status'] = 1

	def get_code(self):
		"""
		The method to get the code

		Returns:
			Choice: An instance of Choice
		"""

		return self.__code

	def set_code(self, code):
		"""
		The method to set the value to code

		Parameters:
			code (Choice) : An instance of Choice
		"""

		if code is not None and not isinstance(code, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: code EXPECTED TYPE: Choice', None, None)
		
		self.__code = code
		self.__key_modified['code'] = 1

	def get_message(self):
		"""
		The method to get the message

		Returns:
			Choice: An instance of Choice
		"""

		return self.__message

	def set_message(self, message):
		"""
		The method to set the value to message

		Parameters:
			message (Choice) : An instance of Choice
		"""

		if message is not None and not isinstance(message, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: message EXPECTED TYPE: Choice', None, None)
		
		self.__message = message
		self.__key_modified['message'] = 1

	def get_details(self):
		"""
		The method to get the details

		Returns:
			dict: An instance of dict
		"""

		return self.__details

	def set_details(self, details):
		"""
		The method to set the value to details

		Parameters:
			details (dict) : An instance of dict
		"""

		if details is not None and not isinstance(details, dict):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: details EXPECTED TYPE: dict', None, None)
		
		self.__details = details
		self.__key_modified['details'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Escalation(object):
	def __init__(self):
		"""Creates an instance of Escalation"""

		self.__days = None
		self.__status = None
		self.__key_modified = dict()

	def get_days(self):
		"""
		The method to get the days

		Returns:
			int: An int representing the days
		"""

		return self.__days

	def set_days(self, days):
		"""
		The method to set the value to days

		Parameters:
			days (int) : An int representing the days
		"""

		if days is not None and not isinstance(days, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: days EXPECTED TYPE: int', None, None)
		
		self.__days = days
		self.__key_modified['days'] = 1

	def get_status(self):
		"""
		The method to get the status

		Returns:
			string: A string representing the status
		"""

		return self.__status

	def set_status(self, status):
		"""
		The method to set the value to status

		Parameters:
			status (string) : A string representing the status
		"""

		if status is not None and not isinstance(status, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: status EXPECTED TYPE: str', None, None)
		
		self.__status = status
		self.__key_modified['status'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
    from .converter import Converter
    import importlib
    import logging
    import re
    import json

except Exception:
    import importlib
    from .converter import Converter
    import logging
    import re
    import json


class XMLConverter(Converter):

    """
    This class processes the API response object to the POJO object and POJO object to an XML object.
    """

    logger = logging.getLogger('SDKLogger')

    def __init__(self, common_api_handler):

        super().__init__(common_api_handler)

        self.unique_dict = {}

        self.count = 0

        self.common_api_handler = common_api_handler

    def form_request(self, request_instance, pack, instance_number, class_member_detail):

        return None

    def append_to_request(self, request_base, request_object):

        return None

    def get_wrapped_response(self, response, pack):

        return None

    def get_response(self, response, pack):

        return None



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Choice, Constants
except Exception:
	from ..exception import SDKException
	from ..util import Choice, Constants


class RequestWrapper(object):
	def __init__(self):
		"""Creates an instance of RequestWrapper"""

		self.__callback = None
		self.__query = None
		self.__file_type = None
		self.__key_modified = dict()

	def get_callback(self):
		"""
		The method to get the callback

		Returns:
			CallBack: An instance of CallBack
		"""

		return self.__callback

	def set_callback(self, callback):
		"""
		The method to set the value to callback

		Parameters:
			callback (CallBack) : An instance of CallBack
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.bulk_read.call_back import CallBack
		except Exception:
			from .call_back import CallBack

		if callback is not None and not isinstance(callback, CallBack):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: callback EXPECTED TYPE: CallBack', None, None)
		
		self.__callback = callback
		self.__key_modified['callback'] = 1

	def get_query(self):
		"""
		The method to get the query

		Returns:
			Query: An instance of Query
		"""

		return self.__query

	def set_query(self, query):
		"""
		The method to set the value to query

		Parameters:
			query (Query) : An instance of Query
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.bulk_read.query import Query
		except Exception:
			from .query import Query

		if query is not None and not isinstance(query, Query):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: query EXPECTED TYPE: Query', None, None)
		
		self.__query = query
		self.__key_modified['query'] = 1

	def get_file_type(self):
		"""
		The method to get the file_type

		Returns:
			Choice: An instance of Choice
		"""

		return self.__file_type

	def set_file_type(self, file_type):
		"""
		The method to set the value to file_type

		Parameters:
			file_type (Choice) : An instance of Choice
		"""

		if file_type is not None and not isinstance(file_type, Choice):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: file_type EXPECTED TYPE: Choice', None, None)
		
		self.__file_type = file_type
		self.__key_modified['file_type'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Attachment(object):
	def __init__(self):
		"""Creates an instance of Attachment"""

		self.__owner = None
		self.__modified_time = None
		self.__file_name = None
		self.__created_time = None
		self.__size = None
		self.__parent_id = None
		self.__editable = None
		self.__file_id = None
		self.__type = None
		self.__se_module = None
		self.__modified_by = None
		self.__state = None
		self.__id = None
		self.__created_by = None
		self.__link_url = None
		self.__description = None
		self.__category = None
		self.__key_modified = dict()

	def get_owner(self):
		"""
		The method to get the owner

		Returns:
			User: An instance of User
		"""

		return self.__owner

	def set_owner(self, owner):
		"""
		The method to set the value to owner

		Parameters:
			owner (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if owner is not None and not isinstance(owner, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: owner EXPECTED TYPE: User', None, None)
		
		self.__owner = owner
		self.__key_modified['Owner'] = 1

	def get_modified_time(self):
		"""
		The method to get the modified_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__modified_time

	def set_modified_time(self, modified_time):
		"""
		The method to set the value to modified_time

		Parameters:
			modified_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if modified_time is not None and not isinstance(modified_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_time EXPECTED TYPE: datetime', None, None)
		
		self.__modified_time = modified_time
		self.__key_modified['Modified_Time'] = 1

	def get_file_name(self):
		"""
		The method to get the file_name

		Returns:
			string: A string representing the file_name
		"""

		return self.__file_name

	def set_file_name(self, file_name):
		"""
		The method to set the value to file_name

		Parameters:
			file_name (string) : A string representing the file_name
		"""

		if file_name is not None and not isinstance(file_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: file_name EXPECTED TYPE: str', None, None)
		
		self.__file_name = file_name
		self.__key_modified['File_Name'] = 1

	def get_created_time(self):
		"""
		The method to get the created_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__created_time

	def set_created_time(self, created_time):
		"""
		The method to set the value to created_time

		Parameters:
			created_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if created_time is not None and not isinstance(created_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_time EXPECTED TYPE: datetime', None, None)
		
		self.__created_time = created_time
		self.__key_modified['Created_Time'] = 1

	def get_size(self):
		"""
		The method to get the size

		Returns:
			int: An int representing the size
		"""

		return self.__size

	def set_size(self, size):
		"""
		The method to set the value to size

		Parameters:
			size (int) : An int representing the size
		"""

		if size is not None and not isinstance(size, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: size EXPECTED TYPE: int', None, None)
		
		self.__size = size
		self.__key_modified['Size'] = 1

	def get_parent_id(self):
		"""
		The method to get the parent_id

		Returns:
			Record: An instance of Record
		"""

		return self.__parent_id

	def set_parent_id(self, parent_id):
		"""
		The method to set the value to parent_id

		Parameters:
			parent_id (Record) : An instance of Record
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.record import Record
		except Exception:
			from ..record import Record

		if parent_id is not None and not isinstance(parent_id, Record):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: parent_id EXPECTED TYPE: Record', None, None)
		
		self.__parent_id = parent_id
		self.__key_modified['Parent_Id'] = 1

	def get_editable(self):
		"""
		The method to get the editable

		Returns:
			bool: A bool representing the editable
		"""

		return self.__editable

	def set_editable(self, editable):
		"""
		The method to set the value to editable

		Parameters:
			editable (bool) : A bool representing the editable
		"""

		if editable is not None and not isinstance(editable, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: editable EXPECTED TYPE: bool', None, None)
		
		self.__editable = editable
		self.__key_modified['$editable'] = 1

	def get_file_id(self):
		"""
		The method to get the file_id

		Returns:
			string: A string representing the file_id
		"""

		return self.__file_id

	def set_file_id(self, file_id):
		"""
		The method to set the value to file_id

		Parameters:
			file_id (string) : A string representing the file_id
		"""

		if file_id is not None and not isinstance(file_id, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: file_id EXPECTED TYPE: str', None, None)
		
		self.__file_id = file_id
		self.__key_modified['$file_id'] = 1

	def get_type(self):
		"""
		The method to get the type

		Returns:
			string: A string representing the type
		"""

		return self.__type

	def set_type(self, type):
		"""
		The method to set the value to type

		Parameters:
			type (string) : A string representing the type
		"""

		if type is not None and not isinstance(type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: type EXPECTED TYPE: str', None, None)
		
		self.__type = type
		self.__key_modified['$type'] = 1

	def get_se_module(self):
		"""
		The method to get the se_module

		Returns:
			string: A string representing the se_module
		"""

		return self.__se_module

	def set_se_module(self, se_module):
		"""
		The method to set the value to se_module

		Parameters:
			se_module (string) : A string representing the se_module
		"""

		if se_module is not None and not isinstance(se_module, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: se_module EXPECTED TYPE: str', None, None)
		
		self.__se_module = se_module
		self.__key_modified['$se_module'] = 1

	def get_modified_by(self):
		"""
		The method to get the modified_by

		Returns:
			User: An instance of User
		"""

		return self.__modified_by

	def set_modified_by(self, modified_by):
		"""
		The method to set the value to modified_by

		Parameters:
			modified_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if modified_by is not None and not isinstance(modified_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_by EXPECTED TYPE: User', None, None)
		
		self.__modified_by = modified_by
		self.__key_modified['Modified_By'] = 1

	def get_state(self):
		"""
		The method to get the state

		Returns:
			string: A string representing the state
		"""

		return self.__state

	def set_state(self, state):
		"""
		The method to set the value to state

		Parameters:
			state (string) : A string representing the state
		"""

		if state is not None and not isinstance(state, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: state EXPECTED TYPE: str', None, None)
		
		self.__state = state
		self.__key_modified['$state'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			string: A string representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (string) : A string representing the id
		"""

		if id is not None and not isinstance(id, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: str', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_created_by(self):
		"""
		The method to get the created_by

		Returns:
			User: An instance of User
		"""

		return self.__created_by

	def set_created_by(self, created_by):
		"""
		The method to set the value to created_by

		Parameters:
			created_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if created_by is not None and not isinstance(created_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by EXPECTED TYPE: User', None, None)
		
		self.__created_by = created_by
		self.__key_modified['Created_By'] = 1

	def get_link_url(self):
		"""
		The method to get the link_url

		Returns:
			string: A string representing the link_url
		"""

		return self.__link_url

	def set_link_url(self, link_url):
		"""
		The method to set the value to link_url

		Parameters:
			link_url (string) : A string representing the link_url
		"""

		if link_url is not None and not isinstance(link_url, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: link_url EXPECTED TYPE: str', None, None)
		
		self.__link_url = link_url
		self.__key_modified['$link_url'] = 1

	def get_description(self):
		"""
		The method to get the description

		Returns:
			string: A string representing the description
		"""

		return self.__description

	def set_description(self, description):
		"""
		The method to set the value to description

		Parameters:
			description (string) : A string representing the description
		"""

		if description is not None and not isinstance(description, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: description EXPECTED TYPE: str', None, None)
		
		self.__description = description
		self.__key_modified['description'] = 1

	def get_category(self):
		"""
		The method to get the category

		Returns:
			string: A string representing the category
		"""

		return self.__category

	def set_category(self, category):
		"""
		The method to set the value to category

		Parameters:
			category (string) : A string representing the category
		"""

		if category is not None and not isinstance(category, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: category EXPECTED TYPE: str', None, None)
		
		self.__category = category
		self.__key_modified['category'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



from .action_wrapper import ActionWrapper
from .users_operations import UsersOperations, GetUsersParam, GetUsersHeader, GetUserHeader
from .customize_info import CustomizeInfo
from .request_wrapper import RequestWrapper
from .shift import Shift
from .body_wrapper import BodyWrapper
from .api_exception import APIException
from .action_response import ActionResponse
from .response_handler import ResponseHandler
from .success_response import SuccessResponse
from .response_wrapper import ResponseWrapper
from .tab_theme import TabTheme
from .theme import Theme
from .action_handler import ActionHandler
from .user import User
from .territory import Territory
from .info import Info
